__author__ = 'chris'
from unittest import TestCase
import subprocess
import os
import shutil

BASE_DIR = os.path.split(__file__)[0]
DJANGUI_SCRIPT_PATH = os.path.join(BASE_DIR, '..', 'scripts', 'djanguify.py')
DJANGUI_TEST_PROJECT_NAME = 'djangui_project'
DJANGUI_TEST_PROJECT_PATH = os.path.join(BASE_DIR, DJANGUI_TEST_PROJECT_NAME)
DJANGUI_TEST_PROJECT_MANAGE = os.path.join(DJANGUI_TEST_PROJECT_PATH, 'manage.py')
PYTHON_INTERPRETTER = 'python'

env = os.environ
env['DJANGO_SETTINGS_MODULE'] = '{}.settings'.format(DJANGUI_TEST_PROJECT_NAME)
env['TESTING'] = 'True'

class TestProject(TestCase):
    def setUp(self):
        # if old stuff exists, remove it
        if os.path.exists(DJANGUI_TEST_PROJECT_PATH):
            shutil.rmtree(DJANGUI_TEST_PROJECT_PATH)

    def tearDown(self):
        shutil.rmtree(DJANGUI_TEST_PROJECT_PATH)

    def test_bootstrap(self):
        proc = subprocess.Popen([PYTHON_INTERPRETTER, DJANGUI_SCRIPT_PATH, '-p', DJANGUI_TEST_PROJECT_NAME],
                                cwd=BASE_DIR, env=env, stderr=subprocess.PIPE)
        stdout, stderr = proc.communicate()
        self.assertEqual(proc.returncode, 0, msg=stderr)


import json

from django.test import TestCase, RequestFactory
from django.core.urlresolvers import reverse

from . import factories, mixins
from ..views import djangui_celery

class CeleryViews(mixins.ScriptFactoryMixin, TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_celery_results(self):
        request = self.factory.get(reverse('celery_results'))
        user = factories.UserFactory()
        request.user = user
        response = djangui_celery.celery_status(request)
        d = response.content.decode("utf-8")
        self.assertEqual({'anon': [], 'user': []}, json.loads(d))
        job = factories.JobFactory()
        job.save()
        response = djangui_celery.celery_status(request)
        d = json.loads(response.content.decode("utf-8"))
        self.assertEqual(1, len(d['anon']))
        job.user = user
        job.save()
        response = djangui_celery.celery_status(request)
        d = json.loads(response.content.decode("utf-8"))
        # we now are logged in, make sure the job appears under the user jobs
        self.assertEqual(1, len(d['user']))
        from django.contrib.auth.models import AnonymousUser
        user = AnonymousUser()
        request.user = user
        response = djangui_celery.celery_status(request)
        d = json.loads(response.content.decode("utf-8"))
        # test empty response since anonymous users should not see users jobs
        self.assertEqual({'anon': [], 'user': []}, d)


from . version import DJANGO_VERSION, DJ18, DJ17, DJ16

if DJANGO_VERSION < DJ17:
    import json
    from django.http import HttpResponse

    class JsonResponse(HttpResponse):
        """
            JSON response
            # from https://gist.github.com/philippeowagner/3179eb475fe1795d6515
        """
        def __init__(self, content, mimetype='application/json', status=None, content_type=None, **kwargs):
            super(JsonResponse, self).__init__(
                content=json.dumps(content),
                mimetype=mimetype,
                status=status,
                content_type=content_type,
            )
else:
    from django.http import JsonResponse

if DJANGO_VERSION >= DJ18:
    from django.template import Engine
else:
    from django.template import Template
    from django.conf import settings
    try:
        settings.configure()
    except RuntimeError:
        pass

    class Engine(object):

        @staticmethod
        def from_string(code):
            return Template(code)


from ..models import Script
from ..backend import utils

class ScriptFactoryMixin(object):
    def tearDown(self):
        for i in Script.objects.all():
            path = i.script_path.path
            utils.get_storage().delete(path)
            path += 'c' # handle pyc junk
            utils.get_storage().delete(path)
        super(ScriptFactoryMixin, self).tearDown()